@@ -14,25 +14,47 @@ import sys
 from array import array
 from raser.model import Mobility
 from raser.model import Avalanche
+from raser.source import MIPs
 
 #The drift of generated particles
 class CalCurrent:
-    def __init__(self, my_d, my_f, my_g4p, dset):
+    def __init__(self, my_d, my_f, my_g4p, dset, batch=0):
         #mobility related with the magnetic field (now silicon useless)
         self.muhh=1650.0   
         self.muhe=310.0
         self.BB=np.array([0,0,0])
         self.sstep=dset.steplength #drift step
+        print(self.sstep)
         self.kboltz=8.617385e-5 #eV/K
         self.max_drift_len=1e9 #maximum diftlenght [um]
-        self.drift(my_d, my_f, my_g4p)
+        self.parameters(my_g4p, my_d, batch)
+        self.ionized_drift(my_g4p,my_f,my_d)
             
-    def drift(self, my_d, my_f, my_g4p, batch=0):    
+    def parameters(self,my_g4p, my_d, batch): 
+        """" Define the output dictionary """   
         self.d_dic_n = {}
         self.d_dic_p = {}
+        self.events_position(my_g4p, batch) 
+        for n in range(len(self.tracks_p)-1):
+            self.d_dic_n["tk_"+str(n+1)] = [ [] for n in range(5) ]
+            self.d_dic_p["tk_"+str(n+1)] = [ [] for n in range(5) ]
+           
+    def events_position(self,my_g4p, batch):
+        """
+        Description:
+            Events position and energy depositon
+        Parameters:
+        ---------
+        batch : int
+            batch = 0: Single event, choose hit electron
+            batch != 0: Multi event, batch is the number of electron     
+        @Modify:
+        ---------
+            2021/09/13
+        """     
         if batch == 0:
             for j in range(len(my_g4p.p_steps)):
-                if my_g4p.edep_devices[j]>0.2 and batch == 0:
+                if len(my_g4p.p_steps[j])>10 and batch == 0:
                     self.beam_number = j
                     self.tracks_p = my_g4p.p_steps[j]
                     self.tracks_step_edep = my_g4p.energy_steps[j]
@@ -47,76 +69,92 @@ class CalCurrent:
             self.tracks_p = my_g4p.p_steps[batch]
             self.tracks_step_edep = my_g4p.energy_steps[batch]
             self.tracks_t_edep = my_g4p.edep_devices[batch]
-        for n in range(len(self.tracks_p)-1):
-            self.d_dic_n["tk_"+str(n+1)] = [ [] for n in range(5) ]
-            self.d_dic_p["tk_"+str(n+1)] = [ [] for n in range(5) ]    
-        self.ionized_drift(my_g4p,my_f,my_d)
-        
-    def ionized_drift(self,my_g4p,my_f,my_d):
 
+    def ionized_drift(self,my_g4p,my_f,my_d):
+        """
+        Description:
+            The drift simulation of all tracks
+            One track is corresponding to one electron-hole pair 
+        """
         for i in range(len(self.tracks_p)-1):
             self.n_track=i+1
             for j in range(2):
                 if (j==0):
                     self.charg=1 #hole
                 if (j==1):
-                    self.charg=-1 #electron                
-                self.initial_parameter()
-                #generated particles positions
-                self.d_x=self.tracks_p[i+1][0]#initial position
-                self.d_y=self.tracks_p[i+1][1]
-                # the z position need to minus the position from geant4
-                self.d_z=self.tracks_p[i+1][2] - my_g4p.init_tz_device 
-                while (self.end_cond==0):
-                    # electic field of the position
-                    self.e_field = my_f.get_e_field(self.d_x,
-                                                    self.d_y,
-                                                    self.d_z)
-                    # modify this part										
-                    if (self.d_y>=(my_d.l_y-1.0) or self.d_x>=(my_d.l_x-1.0) or self.d_z>=(my_d.l_z-1.0)):
-                        self.end_cond=3  
-                    elif (self.d_y<=(1.0) or self.d_x<=(1.0) or self.d_z<=(1.0)):
-                        self.end_cond=8                    
-                    elif (self.e_field[0]==0 and self.e_field[1]==0 and self.e_field[1]==0):
-                        self.end_cond=9
-                    else:                                                                      
-                        self.delta_p() #delta_poisiton                  
-                        self.drift_v(my_d,my_f) #drift_position                    
-                        self.drift_s_step(my_d) #drift_next_posiiton
-                        #charge_collection
-                        self.wpot = my_f.get_w_p(self.d_cx,self.d_cy,self.d_cz)
-                        delta_Uw = (self.wpot 
-                                   - my_f.get_w_p(self.d_x,self.d_y,self.d_z))
-                        self.charge=self.charg*delta_Uw
-                        if(self.v_drift!=0):
-                            self.d_time=self.d_time+self.sstep*1e-4/self.v_drift
-                            self.path_len+=self.sstep
-                        self.d_x=self.d_cx
-                        self.d_y=self.d_cy
-                        self.d_z=self.d_cz
-                        
-                        self.save_inf_track(my_d)  
-                        self.drift_end_condition()
-                    self.n_step+=1 
-        self.get_current(my_d,my_g4p)
-        
+                    self.charg=-1 #electron 
+                self.loop_electon_hole(my_g4p,my_f,my_d,i)
+        self.get_current(my_d)
 
+    def energy_deposition(self,my_d,j):
+        """" Deposition energy and generate e-h pair """
+        sic_loss_e=self.meter_choose(my_d)
+        n_pairs=self.tracks_step_edep[j]*1e6/sic_loss_e
+        return n_pairs 
+
+    def loop_electon_hole(self,my_g4p,my_f,my_d,i):
+        """
+        Description:
+            Loop and record the induced cuurent of each eletron or holes         
+        Parameters:
+        ---------
+        arg1 : int
+            
+        @Modify:
+        ---------
+            2021/09/13
+        """
+        self.initial_parameter()
+        self.d_x=self.tracks_p[i+1][0]
+        self.d_y=self.tracks_p[i+1][1]
+        self.d_z=self.tracks_p[i+1][2] - my_g4p.init_tz_device 
+        while (self.end_cond == 0):
+            if self.judge_whether_insensor(my_d,my_f):               
+                pass
+            else:                                                                     
+                self.delta_p() #delta_poisiton               
+                self.drift_v(my_d,my_f) #drift_position                   
+                self.drift_s_step(my_d) #drift_next_posiiton
+                self.charge_collection(my_f)             
+                self.save_inf_track(my_d) 
+                self.drift_end_condition()
+            self.n_step+=1 
+        
+        
     def initial_parameter(self):
+        """ initial the parameter in the drift """
         self.end_cond=0
         self.d_time=1.0e-9
         self.path_len=0
         self.n_step=0
         self.charge=0
 
+    def judge_whether_insensor(self,my_d,my_f):
+        """
+        Judge the whether the point(x,y,z) is in the sensor
+        and whether the electric field is zero
+        """
+        # electic field of the position
+        self.e_field = my_f.get_e_field(self.d_x,
+                                        self.d_y,
+                                        self.d_z)
+        # modify this part										
+        if (self.d_y>=(my_d.l_y-1.0) or self.d_x>=(my_d.l_x-1.0) or self.d_z>=(my_d.l_z-1.0)):
+            self.end_cond=3  
+        elif (self.d_y<=(1.0) or self.d_x<=(1.0) or self.d_z<=(1.0)):
+            self.end_cond=8                    
+        elif (self.e_field[0]==0 and self.e_field[1]==0 and self.e_field[1]==0):
+            self.end_cond=9
+        return self.end_cond
+
     def delta_p(self):
-        # magnetic field effect
+        """ sstep(1um) split to three position """
         if(self.charg)>0:
             FF=self.list_add(self.e_field,
                              self.cross(self.e_field,self.BB,self.muhh))
         else:
             FF=self.list_sub(self.e_field,
-                             self.cross(self.e_field,self.BB,self.muhe))
-        
+                             self.cross(self.e_field,self.BB,self.muhe))   
         total_ef = self.root_mean_square(FF)
         if(total_ef!=0):
             self.delta_x=-self.sstep*self.charg*FF[0]/total_ef
@@ -175,14 +213,18 @@ class CalCurrent:
                                     *my_d.temperature*self.s_time)
                 self.dif_x=random.gauss(0.0,s_sigma)*1e4
                 self.dif_y=random.gauss(0.0,s_sigma)*1e4
-                self.dif_z=random.gauss(0.0,s_sigma)*1e4
-                       
+                self.dif_z=random.gauss(0.0,s_sigma)*1e4                       
             else:
                 print("the eletric field is too big, \
                        the multiplication appear. The system shold end. ")
                 sys.exit(0)
 
     def drift_s_step(self,my_d):
+        """" Drift distance
+        d_x: raw position
+        delta_x: eletric field
+        dif_x thermal diffusion
+        """
         # x axis   
         if((self.d_x+self.delta_x+self.dif_x)>=my_d.l_x): 
             self.d_cx = my_d.l_x
@@ -205,7 +247,21 @@ class CalCurrent:
         else:
             self.d_cz = self.d_z+self.delta_z+self.dif_z
 
-    def drift_end_condition(self):    
+    def charge_collection(self,my_f):
+        """ Calculate charge collection """ 
+        self.wpot = my_f.get_w_p(self.d_cx,self.d_cy,self.d_cz)
+        delta_Uw = (self.wpot 
+                    - my_f.get_w_p(self.d_x,self.d_y,self.d_z))
+        self.charge=self.charg*delta_Uw
+        if(self.v_drift!=0):
+            self.d_time=self.d_time+self.sstep*1e-4/self.v_drift
+            self.path_len+=self.sstep
+        self.d_x=self.d_cx
+        self.d_y=self.d_cy
+        self.d_z=self.d_cz
+
+    def drift_end_condition(self): 
+        """ Judge whether the drift loop should end """
         if(self.wpot>(1-1e-5)):
             self.end_cond=1
         if(self.d_x<=0):
@@ -220,8 +276,8 @@ class CalCurrent:
             self.end_cond=7
 
     def save_inf_track(self,my_d):
-        e0 = 1.60217733e-19
-        if(((self.charge<0 and my_d.v_voltage<0) 
+        """ Save the information in the dictionary """
+        if(((self.charge<0 and my_d.v_voltage<0)  
              or (self.charge>0 and my_d.v_voltage>0))): 
             if(self.charg>0):
                 self.d_dic_p["tk_"+str(self.n_track)][0].append(self.d_x)
@@ -236,58 +292,67 @@ class CalCurrent:
                 self.d_dic_n["tk_"+str(self.n_track)][3].append(self.charge)
                 self.d_dic_n["tk_"+str(self.n_track)][4].append(self.d_time)
 
-    def get_current(self,my_d,my_g4p):
-        my_d.positive_cu.Reset()
-        my_d.negtive_cu.Reset()
-        my_d.sum_cu.Reset()
-        self.sum_p_current = []
+    def get_current(self,my_d):
+        """ Charge distribution to total current"""
+        self.reset_start(my_d)
         test_p = ROOT.TH1F("test+","test+",my_d.n_bin,my_d.t_start,my_d.t_end)
         test_n = ROOT.TH1F("test-","test-",my_d.n_bin,my_d.t_start,my_d.t_end)
-        test_sum = ROOT.TH1F("test sum","test sum",
-                             my_d.n_bin,my_d.t_start,my_d.t_end)
         total_pairs=0
-        if (my_d.mater == 1): # silicon carbide
-            sic_loss_e = 8.4 #ev
-        elif (my_d.mater == 0):   # silicon
-            sic_loss_e = 3.6 #ev
-        for j in range(len(self.tracks_p)-1):
-            for i in range(len(self.d_dic_p["tk_"+str(j+1)][2])):
-                test_p.Fill(self.d_dic_p["tk_"+str(j+1)][4][i],
-                            self.d_dic_p["tk_"+str(j+1)][3][i])
-            test_p = self.get_current_his(test_p)           
-            for i in range(len(self.d_dic_n["tk_"+str(j+1)][2])):
-                test_n.Fill(self.d_dic_n["tk_"+str(j+1)][4][i],
-                            self.d_dic_n["tk_"+str(j+1)][3][i])
-            test_n = self.get_current_his(test_n)
+        sic_loss_e=self.meter_choose(my_d)
+        for j in range(len(self.tracks_p)-2):
+            test_p,test_n = self.get_trackspn(my_d, test_p, test_n, j)   
             n_pairs=self.tracks_step_edep[j]*1e6/sic_loss_e
             total_pairs+=n_pairs
             test_p.Scale(n_pairs)
             test_n.Scale(n_pairs)            
             my_d.positive_cu.Add(test_p)
-            my_d.negtive_cu.Add(test_n)
+            my_d.negative_cu.Add(test_n)
             test_p.Reset()
             test_n.Reset()
-        self.laudau_t_pairs = self.tracks_t_edep*1e6/sic_loss_e
+
+        self.landau_t_pairs = self.tracks_t_edep*1e6/sic_loss_e
+        print("landau_t_pairs=%s"%self.landau_t_pairs)
+
         if total_pairs != 0:
-            n_scale = self.laudau_t_pairs/total_pairs
+            n_scale = self.landau_t_pairs/total_pairs
         else:
             n_scale=0
         my_d.sum_cu.Add(my_d.positive_cu)
-        my_d.sum_cu.Add(my_d.negtive_cu)
+        my_d.sum_cu.Add(my_d.negative_cu)
         my_d.sum_cu.Scale(n_scale)
 
-    def get_current_his(self,histo):
+    def reset_start(self,my_d):
+        """ Reset th1f """
+        my_d.positive_cu.Reset()
+        my_d.negative_cu.Reset()
+        my_d.sum_cu.Reset()
+        self.sum_p_current = []
+
+    def meter_choose(self,my_d):
+        """ Judge the material of sensor """
+        if (my_d.mater == 1): # silicon carbide
+            sic_loss_e = 8.4 #ev
+        elif (my_d.mater == 0):   # silicon
+            sic_loss_e = 3.6 #ev
+        return sic_loss_e
+
+    def get_trackspn(self, my_d, test_p, test_n, j):
+        """ Total current of each e-h pair"""
         e0 = 1.60217733e-19
-        hist = ROOT.TH1F()
-        histo.Copy(hist)
-        for i in range(hist.GetNbinsX()):
-            histo.SetBinContent(i, hist.GetBinContent(i) \
-                /((hist.GetXaxis().GetXmax() - hist.GetXaxis().GetXmin()) \
-                / hist.GetNbinsX())*e0)
-        return histo
+        for i in range(len(self.d_dic_p["tk_"+str(j+1)][2])):
+            test_p.Fill(self.d_dic_p["tk_"+str(j+1)][4][i],
+                        self.d_dic_p["tk_"+str(j+1)][3][i]/my_d.t_bin*e0)   
+        for i in range(len(self.d_dic_n["tk_"+str(j+1)][2])):
+            test_n.Fill(self.d_dic_n["tk_"+str(j+1)][4][i],
+                        self.d_dic_n["tk_"+str(j+1)][3][i]/my_d.t_bin*e0)
+        return test_p, test_n
         
 # # # mobility model
 def sic_mobility(charge,aver_e,my_d):
+    """ SiC mobility model 
+    SiC reference: TCAD SIMULATION FOR ALPHA-PARTICLE SPECTROSCOPY USING SIC SCHOTTKY DIODE
+    Si  reference: Signal development in irradiated silicon detectors
+    """
     T=my_d.temperature
     E=aver_e
     Neff=abs(my_d.d_neff)
@@ -334,7 +399,7 @@ def sic_mobility(charge,aver_e,my_d):
 
 class CalCurrent2D:
 
-    def __init__(self,track,fen,det):
+    def __init__(self,track,fen,det,model="lgad2D"):
 
         self.track = track
         self.fen = fen
@@ -368,14 +433,20 @@ class CalCurrent2D:
 
         self.s_gain = 1.
 
+        self.path_len=0
+        self.n_step=0
+
         for n in range(len(track.track_position)):
 
             # initial tracks
-            self.delta_track_info_dic_n["tk_"+str(n+1)] = [ [] for n in range(6) ]   # track_time, track_x, track_y, track_charges, track_current, track_gain
+            self.delta_track_info_dic_n["tk_"+str(n+1)] = [ [] for n in range(6) ]   # track_time, track_x, track_y, track_charge, track_current, track_gain
             self.delta_track_info_dic_p["tk_"+str(n+1)] = [ [] for n in range(6) ] 
 
-        self.cal_current()
-        # self.draw_drift_path(det)
+        self.cal_current(model)
+        self.draw_drift_path(det)
+        if isinstance(track,MIPs):
+            self.landau_scale()
+            self.draw_drift_path(det)
 
     
     def drift_diffusion(self,det,fen):
@@ -385,7 +456,10 @@ class CalCurrent2D:
         self.muhh=1650   # mobility related with the magnetic field (now silicon useless)
         self.muhe=310
         self.BB=np.array([0,0])
-        self.sstep=0.1 # drift step
+        if det.det_width > 100:
+            self.sstep = 1 #drift step
+        else:
+            self.sstep=0.1 # drift step
         self.kboltz=8.617385e-5 # eV/K
         self.max_drift_len=1e9 # maximum driftlength [um]
 
@@ -400,7 +474,7 @@ class CalCurrent2D:
         # magnetic field effect
 
         # for hole
-        if(self.charges)>0:
+        if(self.charge)>0:
             FF=self.e_field+self.muhh*np.cross(self.e_field,self.BB)
         # for electron    
         else:
@@ -409,8 +483,8 @@ class CalCurrent2D:
         # the delta x with electric field and magnetic field for unit e or h
 
         if(np.linalg.norm(FF)!=0):
-            self.delta_x = self.sstep*(self.charges/abs(self.charges))*FF[0]/np.linalg.norm(FF)
-            self.delta_y = self.sstep*(self.charges/abs(self.charges))*FF[1]/np.linalg.norm(FF)
+            self.delta_x = self.sstep*self.charge*FF[0]/np.linalg.norm(FF)
+            self.delta_y = self.sstep*self.charge*FF[1]/np.linalg.norm(FF)
         else:
             self.delta_x=0
             self.delta_y=0 
@@ -431,7 +505,7 @@ class CalCurrent2D:
 
         pos = [self.track_x+self.delta_x, self.track_y+self.delta_y]
 
-        self.drift_velocity = my_mobility.cal_mobility(det, pos, self.charges, ef_value)*ef_value
+        self.drift_velocity = my_mobility.cal_mobility(det, pos, self.charge, ef_value)*ef_value
         # print(self.drift_velocity)
 
         self.e_field = ef
@@ -447,12 +521,14 @@ class CalCurrent2D:
             self.dif_y=0
             self.end_condition = 9
         else:
-            
-            DiffOffField=8*1e4 #V/cm
-            #print("ef_value = "+str(ef_value))
+            if(det.mat_name=='Si'):
+                DiffOffField=8*1e4 #V/cm
+            if(det.mat_name=='SiC'):
+                DiffOffField=100*1e4 #V/cm
+                
             if(ef_value<DiffOffField):
                 self.s_time=self.sstep*1e-4/self.drift_velocity
-                s_sigma= math.sqrt(2*self.kboltz*my_mobility.cal_mobility(det, pos, self.charges, ef_value)*det.temperature*self.s_time)
+                s_sigma= math.sqrt(2*self.kboltz*my_mobility.cal_mobility(det, pos, self.charge, ef_value)*det.temperature*self.s_time)
                 self.dif_x=random.gauss(0,s_sigma)*1e4
                 self.dif_y=random.gauss(0,s_sigma)*1e4
 
@@ -464,15 +540,15 @@ class CalCurrent2D:
         #
         # multiplication
         #
-        my_avalanche = Avalanche('vanOverstraeten')
-        tmp_coefficient = my_avalanche.cal_coefficient(ef_value,self.charges,det.temperature)
+        my_avalanche = Avalanche(det.par_dict['Avalanche'])
+        tmp_coefficient = my_avalanche.cal_coefficient(ef_value,self.charge,det.temperature)
 
         self.s_gain = math.exp(self.sstep*1e-4*tmp_coefficient)
 
 
     def update_track_info(self):
 
-        if(self.charges>0):
+        if(self.charge>0):
             self.delta_track_info_dic_p["tk_"+str(self.track_number)][0].append(self.track_time)
             self.delta_track_info_dic_p["tk_"+str(self.track_number)][1].append(self.track_x)
             self.delta_track_info_dic_p["tk_"+str(self.track_number)][2].append(self.track_y)
@@ -480,7 +556,7 @@ class CalCurrent2D:
             self.delta_track_info_dic_p["tk_"+str(self.track_number)][4].append(self.track_current)
             self.delta_track_info_dic_p["tk_"+str(self.track_number)][5].append(self.track_gain)
  
-        if(self.charges<0):
+        if(self.charge<0):
             self.delta_track_info_dic_n["tk_"+str(self.track_number)][0].append(self.track_time)
             self.delta_track_info_dic_n["tk_"+str(self.track_number)][1].append(self.track_x)
             self.delta_track_info_dic_n["tk_"+str(self.track_number)][2].append(self.track_y)
@@ -499,7 +575,7 @@ class CalCurrent2D:
         self.track_time = self.track_time + self.s_time
         self.track_x = self.track_x + self.delta_x + self.dif_x
         self.track_y = self.track_y + self.delta_y + self.dif_y
-        self.track_charges = self.charges
+        self.track_charges = self.charge
         self.track_gain *= self.s_gain
 
         if(self.track_x>=det.det_width):
@@ -516,22 +592,22 @@ class CalCurrent2D:
         
     def update_end_condition(self):
 
-        # if(self.we_field>(1-1e-5)):
-        #     self.end_condition=1
+        if(self.we_field.all()>(1-1e-5)):
+            self.end_condition=1
         if(self.track_x<=0):
             self.end_condition=2
         if(self.track_y<=0):
             self.end_condition=3
-        # else: 
-        #     self.end_condition=0
+        else: 
+            self.end_condition=0
             
-        # if(self.path_len>self.max_drift_len):
-        #     self.end_condition=6
-        # if(self.n_step>10000):
-        #     self.end_condition=7
+        if(self.path_len>self.max_drift_len):
+            self.end_condition=5
+        if(self.n_step>10000):
+            self.end_condition=7
         
 
-    def cal_current(self):
+    def cal_current(self,model):
 
         e0 = 1.60217733e-19
         track = self.track
@@ -540,19 +616,23 @@ class CalCurrent2D:
         #
         # initial carrier track
         #
-        track.mips_ionized()
         
         for i in range(len(track.track_position)):
 
+            if track.ionized_pairs[i]<=0:
+                continue
+
             self.track_number = i+1
 
             for j in range(2):
 
                 if(j==0):
-                    self.charges=1*track.ionized_pairs[i] # hole
+                    self.charge = 1
+                    #self.charges=1*track.ionized_pairs[i] # hole
 
                 if(j==1):
-                    self.charges=-1*track.ionized_pairs[i] # electron
+                    self.charge = -1
+                    #self.charges=-1*track.ionized_pairs[i] # electron
                 
                 self.track_time = 0.
                 self.track_x = track.track_position[i][0]
@@ -587,120 +667,23 @@ class CalCurrent2D:
 
                         self.drift_diffusion(det,fen)
 
+                        if (self.drift_velocity!=0):
+                            self.path_len+=self.sstep
                         # SR current
 
-                        self.track_current = abs(self.charges*e0*self.drift_velocity*wef_value)
+                        self.track_current = abs(self.charge*e0*self.drift_velocity*wef_value)
 
                         self.update_track_info()
                         self.update_step(det)
                         self.update_end_condition()
-
-
-        #
-        # gian carrier track
-        #
-
-        # get gain tracks start info
-        self.gain_track_info_list = [] #[[name,time,x,y,charges]]
-
-        for i in range(len(track.track_position)):
-
-            for j in range(len(self.delta_track_info_dic_p["tk_"+str(i+1)][0])):
-                
-                if(self.delta_track_info_dic_p["tk_"+str(i+1)][5][j]>1.0):
-
-                    tmp_gain_time = self.delta_track_info_dic_p["tk_"+str(i+1)][0][j]
-                    tmp_gain_x = self.delta_track_info_dic_p["tk_"+str(i+1)][1][j]
-                    tmp_gain_y = self.delta_track_info_dic_p["tk_"+str(i+1)][2][j]
-                    tmp_gain_pairs = abs(self.delta_track_info_dic_p["tk_"+str(i+1)][3][j]*(np.max(self.delta_track_info_dic_p["tk_"+str(i+1)][5])-1))
-                    tmp_gain_current = 0.
-
-                    self.gain_track_info_list.append(["tk_"+str(i+1)+"_p_n",tmp_gain_time,tmp_gain_x,tmp_gain_y,-tmp_gain_pairs])
-
-                    self.gain_track_info_list.append(["tk_"+str(i+1)+"_p_p",tmp_gain_time,tmp_gain_x,tmp_gain_y,tmp_gain_pairs])
-
-                    self.delta_gain_track_info_dic["tk_"+str(i+1)+"_p_n"] = [ [] for n in range(5) ]
-                    self.delta_gain_track_info_dic["tk_"+str(i+1)+"_p_p"] = [ [] for n in range(5) ]
-
-                    break
-
-            for k in range(len(self.delta_track_info_dic_n["tk_"+str(i+1)][0])):
-
-                if(self.delta_track_info_dic_n["tk_"+str(i+1)][5][k]>1.0):
-
-                    tmp_gain_time = self.delta_track_info_dic_n["tk_"+str(i+1)][0][k]
-                    tmp_gain_x = self.delta_track_info_dic_n["tk_"+str(i+1)][1][k]
-                    tmp_gain_y = self.delta_track_info_dic_n["tk_"+str(i+1)][2][k]
-                    tmp_gain_pairs = abs(self.delta_track_info_dic_n["tk_"+str(i+1)][3][k]*(np.max(self.delta_track_info_dic_n["tk_"+str(i+1)][5])-1))
-                    tmp_gain_current = 0.
-
-                    self.gain_track_info_list.append(["tk_"+str(i+1)+"_n_n",tmp_gain_time,tmp_gain_x,tmp_gain_y,-tmp_gain_pairs])
-
-                    self.gain_track_info_list.append(["tk_"+str(i+1)+"_n_p",tmp_gain_time,tmp_gain_x,tmp_gain_y,tmp_gain_pairs])
-
-                    self.delta_gain_track_info_dic["tk_"+str(i+1)+"_n_n"] = [ [] for n in range(5) ]
-                    self.delta_gain_track_info_dic["tk_"+str(i+1)+"_n_p"] = [ [] for n in range(5) ]
-
-                    break
-
-        #
-        # cal gain current
-        #
-
-        for i in range(len(self.gain_track_info_list)):
-
-            self.charges = self.gain_track_info_list[i][4]
-
-            self.track_name = self.gain_track_info_list[i][0]
-            self.track_time = self.gain_track_info_list[i][1]
-            self.track_x = self.gain_track_info_list[i][2]
-            self.track_y = self.gain_track_info_list[i][3]
-            self.track_charges = self.gain_track_info_list[i][4]
-            self.track_current = 0.
-
-            self.end_condition = 0
-            while(self.end_condition==0):
-
-                if(self.track_y>=(det.det_thin-1) or self.track_x>=(det.det_width-1)):
-                     
-                    self.end_condition=4
-
-                else:
-                    efx = fen.cal_point_field(self.track_x, self.track_y,fen.electric_field_x_value)
-                    efy = fen.cal_point_field(self.track_x, self.track_y,fen.electric_field_y_value)
-                    ef = np.array([efx,efy])
-                    ef_value = np.linalg.norm(ef)*1e4
-                    self.e_field = np.array([efx,efy])
-                    wefx = fen.cal_point_field(self.track_x, self.track_y,fen.weighting_electric_field_x_value)
-                    wefy = fen.cal_point_field(self.track_x, self.track_y,fen.weighting_electric_field_y_value)
-                    wef = np.array([wefx,wefy])
-                    wef_value = np.linalg.norm(wef)*1e4
-                    self.we_field = np.array([wefx,wefy])
-                                                   
-                    self.drift_diffusion(det,fen)
-                    # SR current
-                    self.track_current = abs(self.charges*e0*self.drift_velocity*wef_value)
-
-                    self.update_gain_track_info()
-                    self.update_step(det)
-                    self.update_end_condition()            
-
+                    self.n_step +=1
 
         det.positive_cu.Reset()
-        det.negtive_cu.Reset()
-
-        det.gain_positive_cu.Reset()
-        det.gain_negtive_cu.Reset()
-
-        det.gain_n_n_cu.Reset()
-        det.gain_n_p_cu.Reset()
-        det.gain_p_n_cu.Reset()
-        det.gain_p_p_cu.Reset()
-
+        det.negative_cu.Reset()
         det.sum_cu.Reset()
 
         temp_positive_cu = ROOT.TH1F("temp+","temp+",det.n_bin,0,det.t_end)
-        temp_negitive_cu = ROOT.TH1F("temp-","temp-",det.n_bin,0,det.t_end)
+        temp_negative_cu = ROOT.TH1F("temp-","temp-",det.n_bin,0,det.t_end)
         temp_sum_cu = ROOT.TH1F("temp_sum","temp_sum",det.n_bin,0,det.t_end)
         
         #
@@ -710,73 +693,183 @@ class CalCurrent2D:
 
             for j in range(len(self.delta_track_info_dic_p["tk_"+str(i+1)][0])):
                 temp_positive_cu.Fill(self.delta_track_info_dic_p["tk_"+str(i+1)][0][j], self.delta_track_info_dic_p["tk_"+str(i+1)][4][j])
+            temp_positive_cu.Scale(track.ionized_pairs[i])
 
             for k in range(len(self.delta_track_info_dic_n["tk_"+str(i+1)][0])):
-                temp_negitive_cu.Fill(self.delta_track_info_dic_n["tk_"+str(i+1)][0][k], self.delta_track_info_dic_n["tk_"+str(i+1)][4][k])
+                temp_negative_cu.Fill(self.delta_track_info_dic_n["tk_"+str(i+1)][0][k], self.delta_track_info_dic_n["tk_"+str(i+1)][4][k])
+            temp_negative_cu.Scale(track.ionized_pairs[i])
 
             det.positive_cu.Add(temp_positive_cu)
-            det.negtive_cu.Add(temp_negitive_cu)
+            det.negative_cu.Add(temp_negative_cu)
 
             temp_positive_cu.Reset()
-            temp_negitive_cu.Reset()
-
-        #
-        # gain current
-        #
-
-        temp_gain_cu = ROOT.TH1F("temp_gain","temp_gain",det.n_bin,0,det.t_end)
-
-        for i in range(len(self.gain_track_info_list)):
-
-            tmp_track_name = self.gain_track_info_list[i][0]
-
-            for j in range(len(self.delta_gain_track_info_dic[tmp_track_name][0])):
-                
-                temp_gain_cu.Fill(self.delta_gain_track_info_dic[tmp_track_name][0][j],self.delta_gain_track_info_dic[tmp_track_name][4][j])
-
-            if(tmp_track_name[-1]=='p'):
-                det.gain_positive_cu.Add(temp_gain_cu)
-
-            if(tmp_track_name[-1]=='n'):
-                det.gain_negtive_cu.Add(temp_gain_cu)
-
-
-            if(tmp_track_name[-3:]=='n_n'):
-                det.gain_n_n_cu.Add(temp_gain_cu)
-
-            if(tmp_track_name[-3:]=='n_p'):
-                det.gain_n_p_cu.Add(temp_gain_cu)
-
-            # if(tmp_track_name[-3:]=='p_n'):
-            #     det.gain_p_n_cu.Add(temp_gain_cu)
-# 
-            # if(tmp_track_name[-3:]=='p_p'):
-            #     det.gain_p_p_cu.Add(temp_gain_cu)
-
-
-            temp_gain_cu.Reset()
-
+            temp_negative_cu.Reset()
 
         det.sum_cu.Add(det.positive_cu)
-        det.sum_cu.Add(det.negtive_cu)
-        det.sum_cu.Add(det.gain_positive_cu)
-        det.sum_cu.Add(det.gain_negtive_cu)
+        det.sum_cu.Add(det.negative_cu)
 
-        #laudau_pairs = track.mips_laudau()
-        laudau_pairs = 4000
+        if model == "lgad2D":
+            #
+            # gain carrier track
+            #
 
-        current_scale = laudau_pairs/track.ionized_total_pairs
+            # get gain tracks start info
+            self.gain_track_info_list = [] #[[name,time,x,y,charge,pairs]]
 
-        det.positive_cu.Scale(current_scale)
-        det.negtive_cu.Scale(current_scale)
-        det.gain_positive_cu.Scale(current_scale)
-        det.gain_negtive_cu.Scale(current_scale)
+            for i in range(len(track.track_position)):
 
-        det.gain_n_n_cu.Scale(current_scale)
-        det.gain_n_p_cu.Scale(current_scale)
-        det.gain_p_n_cu.Scale(current_scale)
-        det.gain_p_p_cu.Scale(current_scale)
-        det.sum_cu.Scale(current_scale)
+                for j in range(len(self.delta_track_info_dic_p["tk_"+str(i+1)][0])):
+                    
+                    if(self.delta_track_info_dic_p["tk_"+str(i+1)][5][j]>1.0):
+
+                        tmp_gain_time = self.delta_track_info_dic_p["tk_"+str(i+1)][0][j]
+                        tmp_gain_x = self.delta_track_info_dic_p["tk_"+str(i+1)][1][j]
+                        tmp_gain_y = self.delta_track_info_dic_p["tk_"+str(i+1)][2][j]
+                        tmp_gain_pairs = abs(self.delta_track_info_dic_p["tk_"+str(i+1)][3][j])
+                        tmp_pairs = track.ionized_pairs[i]*(np.max(self.delta_track_info_dic_n["tk_"+str(i+1)][5])-1)
+                        tmp_gain_current = 0.
+
+                        self.gain_track_info_list.append(["tk_"+str(i+1)+"_p_n",tmp_gain_time,tmp_gain_x,tmp_gain_y,-tmp_gain_pairs,tmp_pairs])
+
+                        self.gain_track_info_list.append(["tk_"+str(i+1)+"_p_p",tmp_gain_time,tmp_gain_x,tmp_gain_y,tmp_gain_pairs,tmp_pairs])
+
+                        self.delta_gain_track_info_dic["tk_"+str(i+1)+"_p_n"] = [ [] for n in range(5) ]
+                        self.delta_gain_track_info_dic["tk_"+str(i+1)+"_p_p"] = [ [] for n in range(5) ]
+
+                        break
+
+                for k in range(len(self.delta_track_info_dic_n["tk_"+str(i+1)][0])):
+
+                    if(self.delta_track_info_dic_n["tk_"+str(i+1)][5][k]>1.0):
+
+                        tmp_gain_time = self.delta_track_info_dic_n["tk_"+str(i+1)][0][k]
+                        tmp_gain_x = self.delta_track_info_dic_n["tk_"+str(i+1)][1][k]
+                        tmp_gain_y = self.delta_track_info_dic_n["tk_"+str(i+1)][2][k]
+                        tmp_gain_pairs = abs(self.delta_track_info_dic_n["tk_"+str(i+1)][3][k])
+                        tmp_pairs = track.ionized_pairs[i]*(np.max(self.delta_track_info_dic_n["tk_"+str(i+1)][5])-1)
+                        tmp_gain_current = 0.
+
+                        self.gain_track_info_list.append(["tk_"+str(i+1)+"_n_n",tmp_gain_time,tmp_gain_x,tmp_gain_y,-tmp_gain_pairs,tmp_pairs])
+
+                        self.gain_track_info_list.append(["tk_"+str(i+1)+"_n_p",tmp_gain_time,tmp_gain_x,tmp_gain_y,tmp_gain_pairs,tmp_pairs])
+
+                        self.delta_gain_track_info_dic["tk_"+str(i+1)+"_n_n"] = [ [] for n in range(5) ]
+                        self.delta_gain_track_info_dic["tk_"+str(i+1)+"_n_p"] = [ [] for n in range(5) ]
+
+                        break
+
+            #
+            # cal gain current
+            #
+
+            for i in range(len(self.gain_track_info_list)):
+
+                self.charge = self.gain_track_info_list[i][4]
+
+                self.track_name = self.gain_track_info_list[i][0]
+                self.track_time = self.gain_track_info_list[i][1]
+                self.track_x = self.gain_track_info_list[i][2]
+                self.track_y = self.gain_track_info_list[i][3]
+                self.track_charges = self.gain_track_info_list[i][4]
+                self.track_current = 0.
+
+                self.n_step = 0
+                self.path_len = 0
+                self.end_condition = 0
+                while(self.end_condition==0):
+
+                    if(self.track_y>=(det.det_thin-1) or self.track_x>=(det.det_width-1)):
+                        
+                        self.end_condition=4
+
+                    else:
+                        efx = fen.cal_point_field(self.track_x, self.track_y,fen.electric_field_x_value)
+                        efy = fen.cal_point_field(self.track_x, self.track_y,fen.electric_field_y_value)
+                        ef = np.array([efx,efy])
+                        ef_value = np.linalg.norm(ef)*1e4
+                        self.e_field = np.array([efx,efy])
+                        wefx = fen.cal_point_field(self.track_x, self.track_y,fen.weighting_electric_field_x_value)
+                        wefy = fen.cal_point_field(self.track_x, self.track_y,fen.weighting_electric_field_y_value)
+                        wef = np.array([wefx,wefy])
+                        wef_value = np.linalg.norm(wef)*1e4
+                        self.we_field = np.array([wefx,wefy])
+                                                    
+                        self.drift_diffusion(det,fen)
+
+                        if (self.drift_velocity!=0):
+                            self.path_len+=self.sstep
+                        # SR current
+                        self.track_current = abs(self.charge*e0*self.drift_velocity*wef_value)
+
+                        self.update_gain_track_info()
+                        self.update_step(det)
+                        self.update_end_condition()
+                    self.n_step+=1
+
+            det.gain_positive_cu.Reset()
+            det.gain_negative_cu.Reset()
+
+            det.gain_n_n_cu.Reset()
+            det.gain_n_p_cu.Reset()
+            det.gain_p_n_cu.Reset()
+            det.gain_p_p_cu.Reset()
+
+            #
+            # gain current
+            #
+
+            temp_gain_cu = ROOT.TH1F("temp_gain","temp_gain",det.n_bin,0,det.t_end)
+
+            for i in range(len(self.gain_track_info_list)):
+
+                tmp_track_name = self.gain_track_info_list[i][0]
+
+                for j in range(len(self.delta_gain_track_info_dic[tmp_track_name][0])):
+                    
+                    temp_gain_cu.Fill(self.delta_gain_track_info_dic[tmp_track_name][0][j],self.delta_gain_track_info_dic[tmp_track_name][4][j])
+                temp_gain_cu.Scale(self.gain_track_info_list[i][5])
+
+                if(tmp_track_name[-1]=='p'):
+                    det.gain_positive_cu.Add(temp_gain_cu)
+
+                if(tmp_track_name[-1]=='n'):
+                    det.gain_negative_cu.Add(temp_gain_cu)
+
+
+                if(tmp_track_name[-3:]=='n_n'):
+                    det.gain_n_n_cu.Add(temp_gain_cu)
+
+                if(tmp_track_name[-3:]=='n_p'):
+                    det.gain_n_p_cu.Add(temp_gain_cu)
+
+                if(tmp_track_name[-3:]=='p_n'):
+                    det.gain_p_n_cu.Add(temp_gain_cu)
+                    
+                if(tmp_track_name[-3:]=='p_p'):
+                    det.gain_p_p_cu.Add(temp_gain_cu)
+
+
+                temp_gain_cu.Reset()
+
+            det.sum_cu.Add(det.gain_positive_cu)
+            det.sum_cu.Add(det.gain_negative_cu)
+
+    def landau_scale(self):
+        #landau_pairs = track.mips_landau()
+        landau_pairs = 4000
+
+        current_scale = landau_pairs/self.track.ionized_total_pairs
+
+        self.det.positive_cu.Scale(current_scale)
+        self.det.negative_cu.Scale(current_scale)
+        self.det.gain_positive_cu.Scale(current_scale)
+        self.det.gain_negative_cu.Scale(current_scale)
+
+        self.det.gain_n_n_cu.Scale(current_scale)
+        self.det.gain_n_p_cu.Scale(current_scale)
+        self.det.gain_p_n_cu.Scale(current_scale)
+        self.det.gain_p_p_cu.Scale(current_scale)
+        self.det.sum_cu.Scale(current_scale)
         
     def draw_drift_path(self,det):
         # ROOT.gStyle.SetOptStat(0)
@@ -850,4 +943,4 @@ class CalCurrent2D:
         c1.cd(2)
         mg2.Draw("APL")
 
-        c1.SaveAs("./fig/silicon_lgad_2D_drift_path_150V.pdf")
\ No newline at end of file
+        c1.SaveAs("./fig/%s_%s_drift_path_%dV.pdf"%(det.mat_name,det.det_model,det.bias_voltage))
